import asyncio
import threading

from requests import Response
from typing import Optional, Iterator
from typing_extensions import Self

from discord_gateway.api import HttpClient
from discord_gateway.constants import ChannelType


class DiscordMessage:

    def __init__(self, data: object | dict, client: HttpClient) -> None:
        if not data:
            return
        from discord_gateway.discord_user import DiscordMember, DiscordUser  # Circular imports
        self.__token = client.get_token()
        data = dict(data.__dict__) if type(data) is not dict else data
        self.data = data
        self.id = data.get('id')
        self.content = data.get('content')
        self.api = client
        if data.get('guild_id', False):
            self.guild = DiscordGuild(guild_id=data.get('guild_id'), api=self.api)
            self.author = DiscordMember(data.get("member", {}), data.get("author", {}), self.guild.id,
                                                     self.api)
        else:
            self.author = DiscordUser(data.get('author', {}), self.api)
        if data.get('channel_id', False):
            self.channel = DiscordChannel(channel_id=self.data.get('channel_id'), api=self.api)

    async def delete(self) -> Response | dict:
        """
        Deletes the message\n
        :return: :class:`Union[Response, dict]`
            The deleted message
        """
        return self.api.requests(method='DELETE', path=f'/channels/{self.channel.id}/messages/{self.id}',
                                 data={})

    async def reply(self, content: str) -> Self:
        """
        Quick way to reply to a message\n
        :param content: :class:`str`
            Message content
        :return: :class:`DiscordMessage`
            The replied message
        """
        payload = {
            'content': content,
            "message_reference":
                {
                    "channel_id": self.channel.id,
                    "message_id": self.id,
                }
        }
        return DiscordMessage(
            self.api.requests(method='POST', path=f'/channels/{self.channel.id}/messages', json=payload),
            client=self.api)

    async def edit(self, content: str) -> Response | dict:
        """
        Edit message\n
        :param content: :class:`str`
            New content
        :return: :class:`Union[Response, dict]`
            New message
        """
        headers = {
            'Authorization': self.api.get_auth(),
            'Content-Type': 'application/json'
        }
        return self.api.requests(method='PATCH', path=f'/channels/{self.channel.id}/messages/{self.id}',
                                 headers=headers, data='{"content":"' + content + '"}')


class DiscordChannel:

    def __init__(self, channel_id: str, api: HttpClient, use_data: bool = False) -> None:
        self.id = channel_id
        self.__token = api.get_token()
        self.api = api
        self._data: dict = {}
        self.name: Optional[str] = None
        if use_data:
            threading.Thread(target=asyncio.run, args=(self.get_data(),)).start()

    def __repr__(self) -> str:
        return f'{self.id}'

    async def get_data(self) -> None:
        self._data = self.api.requests(method='GET', path=f'/channels/{self.id}')
        self.name = self._data.get('name')

    async def send(self, content: str, threads: bool = False) -> DiscordMessage:
        """
        Send message in channel\n
        :param content: :class:`str`
            Message content
        :param threads: :class:`bool`
            Weather to use threads or not
        :return: :class:`DiscordMessage`
            The message
        """
        if content:
            r = self.api.requests(method='POST', path=f'/channels/{self.id}/messages',
                                  data={'content': content}, threads=threads)
            return DiscordMessage(r, client=self.api) if not threads else None

    async def delete(self) -> None:
        """
        Delete the channel\n
        """
        self.api.requests(method='DELETE', path=f'/channels/{self.id}', threads=True)

    async def generate_invite(self, *, max_age: int = 0, max_uses: int = 0, temporary: bool = False) -> str | dict:
        """
        Creates an invitation to the server\n
        :param max_age: :class:`int`
            Max age of the invite before it gets deleted
        :param max_uses: :class:`int`
            Max allowed uses of the invite
        :param temporary: :class:`bool`
            Grants temporary membership on join
        :return: :class:`Union[str, dict]`
            The generated invite
        """
        payload = {
            "max_age": max_age,
            "max_uses": max_uses,
            "temporary": temporary,
        }
        return self.api.requests(method='POST', path=f'/channels/{self.id}/invites', json=payload)

    def messages(self, limit: int = 50) -> Iterator[DiscordMessage]:
        """
        Fetch channel messages\n
        :param limit: :class:`int`
            Amount of messages to fetch
        :return: :class:`DiscordMessage`
            Every message which is included in the provided limit
        """
        r = self.api.requests(method='GET', path=f'/channels/{self.id}/messages', limit=limit)
        for msg in r:
            yield DiscordMessage(msg, client=self.api)


class DiscordGuild:

    def __init__(self, guild_id: str, api: HttpClient) -> None:
        self.id = guild_id
        self.__token = api.get_token()
        self.api = api

    async def channels(self) -> Iterator[DiscordChannel]:
        """
        Get all guild channels\n
        :return: :class:`DiscordChannel`
            The channels of the guild
        """
        for channel in self.api.requests(method='GET', path=f'/guilds/{self.id}/channels'):
            yield DiscordChannel(channel.get('id'), api=self.api, use_data=False)

    async def create_channel(self, name: str, *, channel_type: ChannelType = ChannelType.TextChannel,
                             permission_overwrites: list = (), threads: bool = False) -> DiscordChannel:
        """
        Create a channel in the guild\n
        :param name: :class:`str`
            The name of the channel
        :param channel_type: :class:`ChannelType`
            The channel type
        :param permission_overwrites: :class:`list`
            The permissions of the channel
        :param threads: :class:`bool`
            Weather to use threads or normal async
        :return: :class:`DiscordChannel`
            The created channel
        """
        payload = {
            'name': '-'.join(name.split(' ')),
            'permission_overwrites': permission_overwrites,
            'type': channel_type.value
        }
        return DiscordChannel(
            self.api.requests(method='POST', path=f'/guilds/{self.id}/channels', json=payload, threads=threads).get(
                'id'), api=self.api)

    async def leave(self) -> None:
        """
        Leaves the guild\n
        """
        self.api.requests(method='DELETE', path=f'users/@me/guilds/{self.id}')

    from discord_gateway.discord_user import DiscordMember  # Circular import

    async def kick(self, member: DiscordMember) -> None:
        """
        Kick a member from the guild\n
        :param member: :class:`DiscordMember`
            The member to kick from the guild
        """
        self.api.requests(method='DELETE', path=f'/guilds/{self.id}/members/{member.id}')

    async def ban(self, member: DiscordMember, delete_message_seconds: int = 3600) -> None:
        """
        Ban a member from the guild\n
        :param member: :class:`DiscordMember`
            The member to ban from the guild
        :param delete_message_seconds: :class:`int`
            Delete message seconds, delete all the messages from the user
            in the given time frame
        """
        payload = {
            "delete_message_seconds": delete_message_seconds
        }
        self.api.requests(method='PUT', path=f'/guilds/{self.id}/bans/{member.id}',
                          data=payload)

    async def unban(self, member_id: str) -> None:
        """
        Unbans a member from the guild\n
        :param member_id: :class:`str`
            The member id to unban
        """
        self.api.requests(method='DELETE', path=f'/guilds/{self.id}/bans/{member_id}')



import asyncio
import logging as log
import threading
import types
from inspect import signature
from typing import Callable, Literal
from dataclasses import dataclass

from discord_gateway.api import HttpClient
from discord_gateway.constants import Intents
from discord_gateway.discord_objects import DiscordMessage
from types import SimpleNamespace

from discord_gateway.gateway import GatewayCon, GatewayMessage

LIB_NAME = r"Yoshaa's Getaway"


@dataclass
class GatewayEvent:  # Store events
    function: callable
    name: str
    type: str = 'global'

    def get_args(self, msg: GatewayMessage, token: str, user: SimpleNamespace,
                 is_bot: bool = True) -> DiscordMessage | object | Literal[False]:
        """
        Extracts arguments before passing to an event\n
        :param msg: :class:`GatewayMessage`
            Event message
        :param token: :class:`str`
            Bot token
        :param user: :class:`SimpleNamespace`
            User data
        :param is_bot: :class:`bool`
            Weather the user is a client or a selfBot
        :return:
            Decoded Message
        :rtype:
            DiscordMessage
        """
        args = self.extract_args(msg)
        if self.name == 'message_create':
            if not msg.data.__dict__.get('guild_id', False) or msg.data.author['id'] == user.id:
                log.info('Canceled event')
                return 'cancel'  # Cancel event
            return DiscordMessage(self.extract_args(msg).data, client=HttpClient(token, bot=is_bot)) if args else args
        else:
            return self.extract_args(msg).data if args else args

    def extract_args(self, msg: GatewayMessage) -> GatewayMessage | Literal[False]:
        if len(signature(self.function).parameters):
            return msg
        else:
            return False


class Gateway(GatewayCon):

    def __init__(self, intents: int = Intents.All.value) -> None:
        super().__init__()
        self.user = None
        self.intents: int = intents
        self._handlers: dict[str] = {}

    async def handle_message(self, msg: GatewayMessage) -> None:
        self._seq = msg.sequence
        if msg.op == 1:
            log.info('received heartbeat')
            heartbeat = {
                'op': 1,
                'd': self._seq
            }
            await self.send(heartbeat)
        elif msg.op == 10:
            log.debug('Received Hello!')
            self._pulse = msg.data.heartbeat_interval / 1000
            identify = {
                'op': 2,
                'd': {
                    'token': self._token,
                    'properties': {
                        '$os': 'windows',
                        "$browser": LIB_NAME,
                        "$device": LIB_NAME,
                    },
                }
            }
            if self.is_bot:
                log.debug('Adding intents')
                identify['d']['intents'] = self.intents
            await self.send(identify)
            log.debug('done identify!')
        elif msg.op == 0:
            event = msg.name.lower()
            if event in self._handlers:
                for handler in self._handlers[event]:
                    func = handler.function
                    if handler.type == 'default':
                        if isinstance(func, types.FunctionType):
                            func(msg)
                        else:
                            await func(msg)
                    elif handler.type == 'global':
                        log.info(f'Running event {handler.name}')
                        args = handler.get_args(msg, token=self._token, user=self.user, is_bot=self.is_bot)
                        if args != 'cancel':
                            threading.Thread(target=asyncio.run, args=(func(args) if args else func(),)).start()
                    else:
                        raise Exception(f'Unknown type: {handler.type}')
            else:
                log.warning(f'unhandled event {event}')
        elif msg.op == 11:
            log.debug('Received op 11')
        else:
            raise Exception(f'Unknown op in msg: {msg.op}')

    def add_listener(self, func: Callable, event: str, *, type: str = 'global') -> None:
        """
        Add an event listener, *(the event will be called when
        the bot receives op 10 and the right event name*)\n
        :param func: :class:`Callable`
            Function to execute when the event runs
        :param event: :class:`str`
            The event name to listen to
        :param type: :class:`str`
            Specify default or global event
            __Global__:
                *default for decorators*
                Will be receiving the extracted version of the arguments (not always GatewayMessage)
            __Default:
                Will be receiving the full data (GatewayMessage), used for default functions and listeners
        """
        handler = GatewayEvent(function=func, name=event, type=type)
        if event in self._handlers:
            self._handlers[event].append(handler)
        else:
            self._handlers[event] = [handler]



from types import SimpleNamespace

from requests import Response

from discord_gateway.api import HttpClient
from discord_gateway.discord_objects import DiscordMessage


class UserMessage(DiscordMessage):

    def __init__(self, data: object, client: HttpClient):
        try:  # Try obtaining data
            super().__init__(data, client)
        except AttributeError:
            pass
        self.author = DiscordUser(self.author, self.__token)


class DiscordUser:

    def __init__(self, user, client: HttpClient):
        self.id = user.get("id")
        self.name = user.get("username")
        self.discriminator = user.get("discriminator")
        self.__token = client.get_token()
        self.api = client

    async def send(self, *content) -> UserMessage:
        """
        Send dm to user\n
        :param content: :class:`list[str]`
            The content to send
        :return: :class:`UserMessage`
            The message
        """
        headers = {
            'Authorization': self.api.get_auth(),
            'Content-Type': 'application/json'
        }
        user_id = self.api.requests(method='POST', path=f'/users/@me/channels', json={"recipients": [self.id]},
                                    headers=headers).get('id')
        try:
            if content:
                r = self.api.requests(method='POST', path=f'/channels/{user_id}/messages', data={'content': content})
                return UserMessage(SimpleNamespace(**r), client=self.api)
        except Exception as e:
            raise f'Missing permissions: {e}'


class DiscordMember(DiscordUser):

    def __init__(self, member, user, guild_id, client: HttpClient):
        super().__init__(user, client)
        self.guild_id = guild_id
        self.roles = member.get("roles", [])
        self.nick = member.get("nick", "")
        self.token = client.get_token()

    def add_role(self, role_id: str) -> Response | dict:
        """
        Add a role to member\n
        :param role_id: :class:`str`
            The role id
        :return: :class:`Union[Response, dict]`
            The added role
        """
        return self.api.requests(method='PUT', path=f'/guilds/{self.guild_id}/members/{self.id}/roles/{role_id}')

    def remove_role(self, role_id: str) -> None:
        """
        Remove a role from member\n
        :param role_id: :class:`str`
            The role id
        """
        self.api.requests(method='DELETE', path=f'/guilds/{self.guild_id}/members/{self.id}/roles/{role_id}')

    async def kick(self) -> None:
        """
        Kicks the member\n
        """
        self.api.requests(method='DELETE', path=f'/guilds/{self.id}/members/{self.id}')

    async def ban(self, delete_message_seconds: int = 3600) -> None:
        """
        Bans the member\n
        :param delete_message_seconds: :class:`int`
            Delete message seconds, delete all the messages from the user
            in the given time frame
        """
        payload = {
            "delete_message_seconds": delete_message_seconds
        }
        self.api.requests(method='PUT', path=f'/guilds/{self.id}/bans/{self.id}', data=payload)



import asyncio
import threading
from typing import Optional, Any

import websocket
import json
import traceback

from dataclasses import dataclass
from types import SimpleNamespace

import logging as log

GATEWAY_URL = 'wss://gateway.discord.gg'


@dataclass
class GatewayMessage:
    op: int
    data: object
    sequence: int
    name: str


def decode_msg(msg: str) -> GatewayMessage:
    obj = json.loads(msg)
    op = obj['op']
    data = None
    seq = None
    name = None
    if 'd' in obj:
        try:
            data = SimpleNamespace(**obj['d'])
        except TypeError:
            pass
    if 's' in obj:
        seq = obj['s']
    if 't' in obj:
        name = obj['t']
    return GatewayMessage(op, data, seq, name)


class GatewayCon:
    def __init__(self) -> None:
        log.debug('init Gateway Connection')
        self.is_bot: bool = True
        self._token: str = ''
        self._q = asyncio.Queue()
        self._pulse: float = 1
        self._seq: Optional[int] = None
        self.ws: Optional[websocket.WebSocket] = None

    def run(self, token: str, *, bot: bool = True) -> None:
        """
        Runs the init connection,\n
        Connects to the websocket and sends identify data.\n
        :param token: :class:`str`
            Bot token, access it through the dev portal.
        :param bot: :class:`bool`
            Toggle selfBot or Normal client, used for self botting
        """
        self._token = token
        self.is_bot = bot
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._run_connection())  # Inf loop

    async def _run_connection(self) -> None:
        log.info('Running connection')
        ws_url = f'{GATEWAY_URL}/?v=9&encoding=json'
        self.ws = websocket.WebSocket()
        self.ws.connect(ws_url)
        threading.Thread(target=asyncio.run, args=[self._ping_loop()]).start()  # Keepalive loop
        threading.Thread(target=asyncio.run, args=[self._recv_loop(self.ws)]).start()  # Handler loop

    async def _recv_loop(self, ws: Any) -> None:
        log.debug('Handling msg')
        for msg in ws:
            try:
                decoded = decode_msg(msg)  # Convert to simple namespace
                await self.handle_message(decoded)
            except Exception as e:
                log.error(f'Exception in receive: {e}')
                traceback.print_exc()

    async def _ping_loop(self) -> None:
        while True:
            await asyncio.sleep(self._pulse)
            ping = {'op': 1, 'd': None}
            await self.send(ping)

    async def handle_message(self, msg: GatewayMessage) -> None:
        ...  # Extendable

    async def send(self, msg: dict) -> None:  # Send data to websocket
        try:
            str_msg: json = json.dumps(msg)
            log.debug(f'gateway sent: {msg}')
            self.ws.send(str_msg)
        except json.JSONDecodeError:
            pass  # Ignore json error
        except Exception as e:
            log.error(f'Exception in send: {e}')
            traceback.print_exc()



from typing import Optional

from discord_gateway.api import HttpClient
from discord_gateway.discord_objects import DiscordMessage


class CommandContext(DiscordMessage):

    def __init__(self, data: object | dict, client: HttpClient):
        super().__init__(data, client)
        self.api = client

    async def send(self, content: str, threads: bool = False) -> Optional[DiscordMessage]:
        """
        Faster way to respond to a command\n
        :param content: :class:`str`
            The content to send
        :param threads: :class:`bool`
            Weather to use threads
        :return: :class:`Optional[DiscordMessage]`
            The final message (only works without threading)
        """
        if content:
            r = self.api.requests(method='POST', path=f'/channels/{self.channel.id}/messages',
                                  data={'content': content}, threads=threads)
            return DiscordMessage(r, client=self.api) if not threads else None



from typing import Optional, Callable, Any

from requests import Response

from discord_gateway.discord_objects import DiscordChannel, DiscordGuild
from discord_gateway.discord_user import DiscordUser
from discord_gateway.gateway_handler import Gateway, GatewayMessage
from discord_gateway.api import HttpClient
from types import SimpleNamespace
from discord_gateway.constants import Intents


class GatewayBot(Gateway):

    def __init__(self, *, intents: int = Intents.All):
        super().__init__(intents=intents.value)
        self.api = None
        self.user: Optional[SimpleNamespace] = None
        self.add_listener(self.default_ready, event='ready', type='default')

    async def default_ready(self, msg: GatewayMessage) -> None:
        """
        Default ready,\n
        Stores user data, api and user tag.\n
        :param msg: :class:`GatewayMessage`
            The data received from the identify response
        """
        self.api = HttpClient(self._token, bot=self.is_bot)
        self.user = SimpleNamespace(**msg.data.user)
        self.user.tag = f'{self.user.username}#{self.user.discriminator}'

    def event(self, func: Callable) -> Callable[..., Any]:
        """
        **Decorator**\n
        Used to assign a function with an event\n
        :param func: The function to assign
        :return: :class:`Callable`
            Returns the function itself
        """
        self.add_listener(func, event=func.__name__.replace('on_', '', 1) if func.__name__.startswith(
            'on_') else func.__name__)
        return func

    async def get_channel(self, channel_id: str) -> DiscordChannel:
        """
        Get channel by his id\n
        :param channel_id: :class:`str`
            The channel id
        :return: :class:`DiscordChannel`
            Returns the channel
        """
        return DiscordChannel(channel_id, api=self.api)

    async def get_guild(self, guild_id: str) -> DiscordGuild:
        """
        Get guild by id\n
        :param guild_id: :class:`str`
            The guild id
        :return: :class:`DiscordGuild`
            Returns the guild
        """
        return DiscordGuild(guild_id, api=self.api)

    async def dms(self) -> Response | dict:
        """
        Get open dms of bot\n
        :return: :class:`Union[Response, dict]`
            Returns the open dms
        """
        return self.api.requests(method='GET', path='/users/@me/channels')

    async def get_user(self, user_id: str) -> DiscordUser:
        """
        Get user by his id\n
        :param user_id: :class:`str`
            The user id
        :return: :class:`DiscordUser`
            Returns the user
        """
        headers = {
            'authorization': self.api.get_auth(),
            'content-type': 'application/json'
        }
        return DiscordUser(self.api.requests(method='POST', path=f'/users/@me/channels', json={"recipients": [user_id]},
                                             headers=headers), self.api)

    async def join(self, server_invite: str) -> Response | dict:
        """
        Join a guild by invite {SelfBot only}\n
        :param server_invite: :class:`str`
            The server invite link
        :return: :class:`Union[Response, dict]`
            The obtained server
        """
        return self.api.requests(method='POST', path=f'/invites/{server_invite}')

    async def leave(self, guild_id: str) -> None:
        """
        Leaves a guild\n
        :param guild_id: :class:`str`
            The guild id to leave
        """
        self.api.requests(method='DELETE', path=f'/users/@me/guilds/{guild_id}')

    async def create_group_chat(self, ids: list[str]) -> Response | dict:
        """
        Creates a group chat with members {SelfBot only}\n
        :param ids: :class:`list[str]`
            The list of the member ids
        :return: :class:`Union[Response, dict]`
            The opened group chat
        """
        headers = {
            'authorization': self.api.get_auth(),
            'content-type': 'application/json'
        }
        return self.api.requests(method='POST', path=f'/users/@me/channels', json={'recipients': ids}, headers=headers)



from __future__ import annotations

import asyncio
import threading
from inspect import signature
from typing import Callable

from .command_context import CommandContext
from discord_gateway import GatewayBot, GatewayMessage

import logging as log


class CommandsBot(GatewayBot):
    def __init__(self, *args, command_prefix: str, case_sensitivity: bool = False,
                 notify_errors: bool = False, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.case_sensitivity = case_sensitivity
        self.prefix = command_prefix
        self.__commands: dict = {}
        self.add_listener(self.get_message, event='message_create', type='default')
        self.notify_errors = notify_errors

    async def get_message(self, msg: GatewayMessage) -> None:
        """
        Default on_message_create,\n
        register messages as commands\n
        :param msg: :class:`GatewayMessage`
            The message to check for
        """
        if not msg.data.content.startswith(self.prefix) or not msg.data.__dict__.get('guild_id', False) or \
                msg.data.author['id'] == self.user.id:
            return
        self.register_commands(CommandContext(msg.data, self.api))

    def command(self, aliases: list = ()) -> Callable[[{__name__}], None]:
        """
        **Decorator**\n
        Used to assign a function with a command\n
        :param aliases: :class:`list[str]`
            Aliases for the given command
        :return: :class:`Callable[[{__name__}], None]`
            The nested function
        """

        def decorator(command: Callable) -> None:
            """
            **Nested Decorator**\n
            Takes a function and assign with a command\n
            :param command: :class:`Callable`
                The function to assign
            """
            self.__commands[command.__name__] = command
            for alias in aliases:
                self.__commands[str(alias)] = command

        return decorator

    def register_commands(self, msg: CommandContext) -> None:
        """
        Checks for available assigned commands\n
        if true:\n
        \tExecutes the commands\n
        if false:\n
        \tIgnores or raises an error depends on your settings\n
        :param msg: :class:`CommandContext`
            The command, data
        """
        command_name = msg.content.replace(self.prefix, '', 1).split(' ')[0]
        if command_name in self.__commands:
            func = self.__commands[command_name]
            sig = signature(func).parameters
            if not any([sign.kind == sign.VAR_POSITIONAL for sign in sig.values()]):
                non_default = filter(lambda sign: sign.default is sign.empty, sig.values())
                args = msg.content.split(' ')[1:len(sig)]
                if len(list(non_default)) - 1 > len(args):
                    print(f'Error: Command {command_name} missing required arguments')
                    return
            else:
                args = msg.content.replace(self.prefix, '', 1).split(' ')[1:]
            log.info(f'running command: {command_name}({", ".join(args)})')
            args = [str(sign) if sign.annotation is sign.empty else sign.annotation(arg) for arg, sign in
                    zip(args, list(sig.values())[1:])]
            threading.Thread(target=asyncio.run, args=(func(msg, *args),)).start()
        elif self.notify_errors:
            print(f'Warning: Unknown command {command_name}')



from enum import Enum

Events: list[str] = ['ready', '']


class Intents(Enum):  # Discord intents
    Guilds: int = 1 << 0
    GuildMembers: int = 1 << 1
    GuildBans: int = 1 << 2
    GuildEmojiAndStickers: int = 1 << 3
    GuildIntegrations: int = 1 << 4
    GuildWebhooks: int = 1 << 5
    GuildInvites: int = 1 << 6
    GuildVoiceStates: int = 1 << 7
    GuildPresences: int = 1 << 8
    GuildMessages: int = 1 << 9
    GuildMessageReactions: int = 1 << 10
    GuildStartTyping: int = 1 << 11
    DirectMessages: int = 1 << 12
    DirectMessageReactions: int = 1 << 13
    DirectMessageTyping: int = 1 << 14
    MessageContent: int = 1 << 15
    GuildScheduledEvents: int = 1 << 16
    AutoModerationConfiguration: int = 1 << 20
    AutoModerationExecution: int = 1 << 21
    GuildsAll: int = 69631
    DirectMessagesAll: int = 28672
    AutoModerationAll: int = 3145728
    All: int = 3244031


class ChannelType(Enum):
    TextChannel: int = 0
    VoiceChannel: int = 2



"""
Gateway __init__
"""

__title__ = 'DiscordGateway'
__author__ = 'zYoshaa'
__version__ = '1.0.1'

import logging as log
from typing import NamedTuple, Literal

from .api import *
from .constants import *
from .discord_objects import *
from .discord_user import *
from .gatewaybot import *
from .gateway_handler import *
from .gateway import *


log.basicConfig(encoding='utf-8', level=log.ERROR)
